from django import VERSION as DJANGO_VERSION
from django.test import TestCase

if DJANGO_VERSION[:2] < (1, 10):
    from django.core.urlresolvers import reverse
else:
    from django.urls import reverse
from djeddit.models import Topic, Thread, Post
from slugify import slugify

# Create your tests here.


class TopicModelTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.topic = Topic.objects.create(title='Test Topic')
        # create 3 threads
        cls.threadCount = 3
        cls.threads = [Thread.objects.create(title='Thread %s' % i,
                                             topic=cls.topic,
                                             op=Post.objects.create()) for i in range(1, cls.threadCount + 1)]

    def testThreadCount(self):
        self.assertEqual(self.topic.getThreadCount(), self.threadCount)

    def testUrlTitle(self):
        self.assertEqual(self.topic.urlTitle, 'Test-Topic')

    def testGetTopic(self):
        self.assertEqual(Topic.getTopic('Test Topic'), self.topic)
        self.assertEqual(Topic.getTopic('Test-Topic'), self.topic)


class ThreadModelTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.topic = Topic.objects.create(title='Test Topic')
        cls.thread = Thread.objects.create(title='Test Thread', topic=cls.topic, op=Post.objects.create())

    def testThreadSlugGeneration(self):
        thread = Thread.objects.create(title='Slug Test 1!', topic=self.topic, op=Post.objects.create())
        slug = slugify(thread.title, to_lower=True, max_length=80)
        self.assertEqual(thread.slug, slug)

    def testThreadRelativeUrl(self):
        threadPageUrl = reverse('threadPage', args=[self.thread.topic.urlTitle, self.thread.id, self.thread.slug])
        self.assertEqual(self.thread.relativeUrl, threadPageUrl)

    def testThreadDelete(self):
        thread = Thread.objects.create(title='temp thread', topic=self.topic, op=Post.objects.create())
        uid = thread.op.uid
        thread.delete()
        self.assertRaises(Thread.DoesNotExist, Thread.objects.get, title=thread.title)
        self.assertRaises(Post.DoesNotExist, Post.objects.get, uid=uid)


class PostModelTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.post = Post.objects.create()
        cls.thread = Thread.objects.create(title='Test Thread', topic=Topic.objects.create(title='Test Topic'), op=cls.post)
        cls.replies = [Post.objects.create(parent=cls.post) for _ in range(1, 4)]

    def testGetReplies(self):
        self.assertListEqual(self.replies, list(self.post.getReplies()))

    def testGetThread(self):
        self.assertEqual(self.thread, self.post.thread)

    def testGetScore(self):
        self.post.upvotes = 2
        self.post.downvotes = 1
        self.post.save()
        self.post.refresh_from_db()
        self.assertEqual(self.post.score, 1)

    def testGetSortedRepliesByWsi(self):
        for i, p in enumerate(self.replies):
            p.upvotes = i
            p.save()
            self.replies[i].refresh_from_db()
        self.assertEqual(tuple(self.post.getSortedReplies(by_wsi=True)), tuple(reversed(self.replies)))

    def testGetSortedRepliesByDate(self):
        self.assertEqual(tuple(self.post.getSortedReplies(by_wsi=False)), tuple(self.replies))



from django.contrib import admin

from .models import Topic
from .models import Post
from .models import Thread
# Register your models here.

@admin.register(Thread)
class ThreadAdmin(admin.ModelAdmin):
    fields = ('title', 'slug', 'locked')
    readonly_fields = ('views', 'topic', 'op')

@admin.register(Post)
class PostAdmin(admin.ModelAdmin):
    fields = ('content', 'modified_on')
    readonly_fields = ('created_by',
                       'created_on',
                       '_upvotes',
                       '_downvotes',
                       'wsi',
                       'parent',
                       'ip_address',
                       'user_agent')

admin.site.register(Topic)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2017-06-15 23:03
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djeddit', '0002_thread_locked'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='post',
            name='score',
        ),
        migrations.AddField(
            model_name='post',
            name='_downvotes',
            field=models.IntegerField(blank=True, default=0),
        ),
        migrations.AddField(
            model_name='post',
            name='_upvotes',
            field=models.IntegerField(blank=True, default=0),
        ),
        migrations.AddField(
            model_name='post',
            name='wsi',
            field=models.FloatField(blank=True, default=0),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2017-05-23 21:47
from __future__ import unicode_literals

from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import djeddit.models
import mptt.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Post',
            fields=[
                ('content', models.TextField(blank=True, default='')),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('modified_on', models.DateTimeField(auto_now=True)),
                ('score', models.IntegerField(blank=True, default=0)),
                ('uid', models.UUIDField(default=djeddit.models.gen_uuid, editable=False, primary_key=True, serialize=False)),
                ('lft', models.PositiveIntegerField(db_index=True, editable=False)),
                ('rght', models.PositiveIntegerField(db_index=True, editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(db_index=True, editable=False)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('parent', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='djeddit.Post')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Thread',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=120)),
                ('url', models.URLField(blank=True, default='', max_length=120)),
                ('views', models.IntegerField(blank=True, default=0)),
                ('op', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='djeddit.Post')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Topic',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=20, unique=True, validators=[django.core.validators.RegexValidator('^[0-9a-zA-Z ]*$', 'Only alphanumeric characters are allowed.')])),
                ('description', models.CharField(blank=True, default='', max_length=120)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='UserPostVote',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('val', djeddit.models.IntegerRangeField(blank=True, default=0)),
                ('post', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='djeddit.Post')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='thread',
            name='topic',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djeddit.Topic'),
        ),
        migrations.AlterField(
            model_name='thread',
            name='title',
            field=models.CharField(max_length=70),
        ),
    ]



from django.test import TestCase
from djeddit.forms import TopicForm, ThreadForm

# Create your tests here.


class TopicFormTest(TestCase):
    def testIsValid(self):
        tf = TopicForm(dict(title='Test Topic'))
        self.assertTrue(tf.is_valid())

    def testEmpty(self):
        tf = TopicForm()
        self.assertTrue(not tf.is_valid())


class ThreadFormTest(TestCase):
    def testIsValid(self):
        tf = ThreadForm(dict(title='Test Thread'))
        self.assertTrue(tf.is_valid())

    def testEmpty(self):
        tf = TopicForm()
        self.assertTrue(not tf.is_valid())



from datetime import datetime
from django import VERSION as DJANGO_VERSION
from django import template
if DJANGO_VERSION[:2] < (1, 10):
    from django.core.urlresolvers import reverse
else:
    from django.urls import reverse
from django.utils import formats
from django.utils.timezone import utc
from djeddit.models import UserPostVote

register = template.Library()


def getAmountContext(num, name, infix=''):
    if num:
        if abs(num) > 1:
            if name.endswith('y'):
                name = '{name}ies'.format(name=name[:-1])
            else:
                name += 's'
        amount_string = '{num} {infix} {name}'.format(num=num,  infix=infix, name=name)
        return ' '.join(amount_string.split())


def getBoolean(condition):
    return 'true' if condition else 'false'

@register.simple_tag
def getAmount(num, name, infix=''):
    amountContext = getAmountContext(num, name, infix=infix)
    if amountContext:
        return amountContext
    amount_string = 'no {infix} {name}s'.format(name=name, infix=infix)
    return ' '.join(amount_string.split())


@register.simple_tag
def postUserName(created_by):
    return created_by or 'guest'


@register.simple_tag
def postScore(score):
    return getAmountContext(score, 'point') if score else 'no score'


@register.simple_tag
def postLevelWidth(thread, post):
    return '1' if all(p != thread.op for p in (post, post.parent)) else '0'


@register.simple_tag
def postWidth(thread, post):
    """:return number of columns to fill"""
    return '11' if all(p != thread.op for p in (post, post.parent)) else '12'


@register.simple_tag
def postVoteClicked(user, post, upvote):
    try:
        userPostVote = UserPostVote.objects.get(user=user, post=post)
        if (userPostVote.val == 1 and upvote) or (userPostVote.val == -1 and not upvote):
            return 'color-primary'
    except UserPostVote.DoesNotExist:
        pass
    return ''

@register.simple_tag
def postVoteOP(thread, post):
    return 'glyphicon-chevron-up-op' if thread.op != post else ''


@register.simple_tag
def postContainer(parent):
    return "post-container" if parent else "op-container"


@register.simple_tag
def postDate(dt, prefix=''):
    dt_now = datetime.utcnow().replace(tzinfo=utc)
    if dt.date() == dt_now.date():
        delta = dt_now - dt
        hours = int(delta.seconds / 3600)
        string = getAmountContext(hours, 'hour')
        if string:
            return '%s ago' % string
        minutes = int(delta.seconds / 60)
        string = getAmountContext(minutes, 'minute')
        if string:
            return '%s ago' % string
        return 'less than a minute ago'
    return prefix + formats.date_format(dt.date(), "DATE_FORMAT")


@register.simple_tag
def threadUrl(thread):
    """:return thread url if it has one, otherwise forward to thread page"""
    return thread.url or thread.relativeUrl

@register.simple_tag
def postUrl(post):
    return '%s/#%s' % (post.thread.relativeUrl, post.uid)

@register.simple_tag
def threadIconClass(thread):
    return 'fa-link' if thread.url else 'fa-commenting-o'

@register.simple_tag
def firstLine(s, max_len=0):
    line = s.split('\n')[0]
    if max_len and len(line) > max_len:
        return '%s...' % line[:max_len]
    return line

@register.simple_tag
def toggleHeader(thread, post, user):
    return getBoolean(user.is_superuser and thread.op == post)


@register.simple_tag
def isOp(thread, post):
    return getBoolean(thread.op == post)

@register.simple_tag
def userStatusSelected(user, status):
    if user.is_active:
        if user.is_superuser:
            return 'selected' if status == 'admin' else ''
        return 'selected' if status == 'active' else ''
    return 'selected' if status == 'banned' else ''

@register.filter
def missingRepliesCount(post, shown_replies):
    excluded_uids = [r.uid for r in shown_replies]
    return post.getReplies(excluded=excluded_uids).count()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2017-12-12 06:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djeddit', '0003_auto_20170616_0203'),
    ]

    operations = [
        migrations.AddField(
            model_name='post',
            name='ip_address',
            field=models.GenericIPAddressField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='post',
            name='user_agent',
            field=models.CharField(blank=True, max_length=150, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2017-05-29 06:35
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djeddit', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='thread',
            name='locked',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-01-17 07:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djeddit', '0004_auto_20171212_1229'),
    ]

    operations = [
        migrations.AddField(
            model_name='thread',
            name='is_stickied',
            field=models.BooleanField(default=False),
        ),
    ]
